using System;
using System.Collections.Generic;
using System.Text;

using System.Net.Sockets;
using System.IO;

namespace SMS
{
    public class EmailMessage
    {
        public string From;
        public string To;
        public string ReplyTo;
        public string Subject;
        public string Date;
        public string ContentType;
        public string Body;

        private string m_MessageSource;

        public EmailMessage(string Message)
        {
            m_MessageSource = Message;
            ParseEmail();
        }

        private void ParseEmail()
        {
            this.ReplyTo = ParseHeader("Reply-To:");
            this.From = ParseHeader("From:");
            this.To = ParseHeader("To:");
            this.Subject = ParseHeader("Subject:");
            this.Date = ParseHeader("Date:");
            this.ContentType = ParseHeader("Content-Type:");
            this.Body = ParseBody();
        }

        private string ParseHeader(string header)
        {
            int s = m_MessageSource.IndexOf(header);
            if (s == -1) return "";
            s = m_MessageSource.IndexOf(':', s) + 1;
            int e = m_MessageSource.IndexOf("\r\n", s);
            if (e == -1) e = m_MessageSource.Length;
            return m_MessageSource.Substring(s, e - s);
        }

        private string ParseBody()
        {
            int s = m_MessageSource.IndexOf("\r\n\r\n");
            if (s == -1) return "";
            return m_MessageSource.Substring(s + 4);
        }


        public string MessageSource
        {
            get
            {
                return m_MessageSource;
            }

            set
            {
                m_MessageSource = value;
                ParseEmail();
            }
        }
    }



    class POP3
    {
        private TcpClient TCP;
        private NetworkStream POP3Stream;
        private StreamReader InStream;
        private string data;
        private string CRLF = "\r\n";
        public POP3(String Server, string user, string password)
        {
            try
            {
                TCP = new TcpClient();
                TCP.Connect(Server, 110);
                POP3Stream = TCP.GetStream();
                InStream = new StreamReader(POP3Stream);


                if (!WaitFor("+OK")) POPerror();
                SendData("USER " + user);
                if (!WaitFor("+OK")) POPerror();
                SendData("PASS " + password);
                if (!WaitFor("+OK")) POPerror();
            }
            catch
            { }
        }

        public void close()
        {
            try
            {
                SendData("QUIT");
                InStream.Close();
                POP3Stream.Close();
            }
            catch
            { }
        }

        Boolean WaitFor(String target)
        {
            do
            {
                data = InStream.ReadLine();
            } while (data.Length == 0);
            return data.StartsWith(target);
        }


        void SendData(string command)
        {
            byte[] outbuff = ASCIIEncoding.ASCII.GetBytes(command + CRLF);
            POP3Stream.Write(outbuff, 0, command.Length + 2);

        }


        public int NumEmails
        {
            get
            {
                try
                {
                    SendData("STAT");
                    if (!WaitFor("+OK")) POPerror();
                    string[] nums = data.Split(' ');
                    return int.Parse(nums[1]);
                }
                catch
                {
                    return 0;
                }
            }
        }


        public void delMail(int n)
        {
            try
            {
                SendData("DELE " + n.ToString());
                if (!WaitFor("+OK")) POPerror();
            }
            catch { }
        }

        public EmailMessage getMail(int n)
        {
            try
            {
                SendData("RETR " + n.ToString());
                String temp;
                StringBuilder mess = new StringBuilder();

                if (!WaitFor("+OK")) POPerror();
                temp = InStream.ReadLine();
                while (temp != ".")
                {
                    mess.Append(temp + CRLF);

                    temp = InStream.ReadLine();
                }

                EmailMessage Emess = new EmailMessage(mess.ToString());
                return Emess;
            }
            catch
            {
                EmailMessage Emess = new EmailMessage("No email was retrieved");
                return Emess;
            }
        }



        void POPerror()
        {
            System.Windows.Forms.MessageBox.Show("POP3 Error");
        }

    }
}
